@@ -12,6 +12,9 @@
 
 """Client for local Quantum experiment services."""
 
+# pylint treats the dataframes as JsonReader for some reason
+# pylint: disable=no-member
+
 import logging
 import os
 import uuid
@@ -24,9 +27,12 @@ import numpy as np
 from qiskit_ibm_experiment.exceptions import (
     IBMExperimentEntryNotFound,
     IBMExperimentEntryExists,
+    IBMApiError,
     RequestsApiError,
 )
 
+from qiskit_ibm_experiment.service.utils import str_to_utc
+
 logger = logging.getLogger(__name__)
 
 
@@ -70,12 +76,11 @@ class LocalExperimentClient:
         """ExperimentClient constructor.
 
         Args:
-            access_token: The session's access token
-            url: The session's base url
-            additional_params: additional session parameters
+            main_dir: The dir in which to place the db files and subdirs
+            local_save: whether to store data to disk or not
         """
-        self._experiments = None
-        self._results = None
+        self._experiments = pd.DataFrame()
+        self._results = pd.DataFrame()
         self._figures = None
         self._files = None
         self._files_list = {}
@@ -125,9 +130,9 @@ class LocalExperimentClient:
                 with open(os.path.join(self.files_dir, filename), "w") as file:
                     file.write(file_data)
 
-    def serialize(self, df):
+    def serialize(self, dataframe):
         """Serializes db values as JSON"""
-        result = df.replace({np.nan: None}).to_dict("records")[0]
+        result = dataframe.replace({np.nan: None}).to_dict("records")[0]
         return json.dumps(result)
 
     def init_db(self):
@@ -213,28 +218,19 @@ class LocalExperimentClient:
         backend_name: Optional[str] = None,
         tags: Optional[List[str]] = None,
         parent_id: Optional[str] = None,
-        tags_operator: Optional[str] = "OR",
         **filters: Any,
     ) -> str:
         """Retrieve experiments, with optional filtering.
 
         Args:
             limit: Number of experiments to retrieve.
-            marker: Marker used to indicate where to start the next query.
-            backend_name: Name of the backend.
-            experiment_type: Experiment type.
-            start_time: A list of timestamps used to filter by experiment start time.
             device_components: A list of device components used for filtering.
+            experiment_type: Experiment type.
+            backend_name: Name of the backend.
             tags: Tags used for filtering.
-            hub: Filter by hub.
-            group: Filter by hub and group.
-            project: Filter by hub, group, and project.
-            exclude_public: Whether or not to exclude experiments with a public share level.
-            public_only: Whether or not to only return experiments with a public share level.
-            exclude_mine: Whether or not to exclude experiments where I am the owner.
-            mine_only: Whether or not to only return experiments where I am the owner.
             parent_id: Filter by parent experiment ID.
-            sort_by: Sorting order.
+            **filters: A set of additional filters/sorters for the results
+            (currently only "start_time" and "sort_by")
 
         Returns:
             A list of experiments and the marker, if applicable.
@@ -268,51 +264,53 @@ class LocalExperimentClient:
             )
 
         if tags is not None:
-            if tags_operator == "OR":
+            if tags[:3] == "any":
+                tags_list = tags[4:].split(",")
                 df = df.loc[
-                    df.tags.apply(lambda dftags: any(x in dftags for x in tags))
+                    df.tags.apply(lambda dftags: any(x in dftags for x in tags_list))
                 ]
-            elif tags_operator == "AND":
+            elif tags[:8] == "contains":
+                tags_list = tags[9:].split(",")
                 df = df.loc[
-                    df.tags.apply(lambda dftags: all(x in dftags for x in tags))
+                    df.tags.apply(lambda dftags: all(x in dftags for x in tags_list))
                 ]
             else:
                 raise ValueError("Unrecognized tags operator")
 
-        if "start_datetime_before" in filters:
-            df = df.loc[df.start_time <= filters["start_datetime_before"]]
-        if "start_datetime_after" in filters:
-            df = df.loc[df.start_time >= filters["start_datetime_after"]]
+        start_datetime_before = None
+        start_datetime_after = None
+        if "start_time" in filters:
+            for start_time_string in filters["start_time"]:
+                string_type = start_time_string[:2]
+                value = str_to_utc(start_time_string[3:])
+                if string_type == "ge":
+                    start_datetime_after = value
+                if string_type == "le":
+                    start_datetime_before = value
+
+        if start_datetime_before is not None:
+            df = df.loc[df.start_time.apply(str_to_utc) <= start_datetime_before]
+        if start_datetime_after is not None:
+            df = df.loc[df.start_time.apply(str_to_utc) >= start_datetime_after]
 
         sort_by = filters.get("sort_by")
         if sort_by is None:
-            sort_by = "start_datetime:desc"
+            sort_by = "start_time:desc"
+        sort_by += ",uuid:asc"
+        sort_by = sort_by.split(",")
 
-        if not isinstance(sort_by, list):
-            sort_by = [sort_by]
-
-        # TODO: support also experiment_type
-        if len(sort_by) != 1:
-            raise ValueError(
-                "The fake service currently supports only sorting by start_datetime"
-            )
-
-        sortby_split = sort_by[0].split(":")
-        # TODO: support also experiment_type
-        if (
-            len(sortby_split) != 2
-            or sortby_split[0] != "start_datetime"
-            or (sortby_split[1] != "asc" and sortby_split[1] != "desc")
-        ):
-            raise ValueError(
-                "The fake service currently supports only sorting by start_datetime, which can be "
-                "either asc or desc"
-            )
-
-        df = df.sort_values(
-            ["start_time", "uuid"], ascending=[(sortby_split[1] == "asc"), True]
-        )
+        sort_by_columns = []
+        sort_by_ascending = []
+        for sort_by_element in sort_by:
+            sortby_split = sort_by_element.split(":")
+            if len(sortby_split) != 2 or (
+                sortby_split[1] != "asc" and sortby_split[1] != "desc"
+            ):
+                raise ValueError(f"Sortby filter {sort_by} is malformed")
+            sort_by_columns.append(sortby_split[0])
+            sort_by_ascending.append(sortby_split[1] == "asc")
 
+        df = df.sort_values(sort_by_columns, ascending=sort_by_ascending)
         df = df.iloc[:limit]
         result = {"experiments": df.replace({np.nan: None}).to_dict("records")}
         return json.dumps(result)
@@ -350,6 +348,10 @@ class LocalExperimentClient:
         data_dict = json.loads(data)
         if "uuid" not in data_dict:
             data_dict["uuid"] = str(uuid.uuid4())
+        if "start_time" not in data_dict:
+            data_dict["start_time"] = str(datetime.now())
+        if "tags" not in data_dict:
+            data_dict["tags"] = []
         exp = self._experiments.loc[self._experiments.uuid == data_dict["uuid"]]
         if not exp.empty:
             raise IBMExperimentEntryExists
@@ -485,7 +487,7 @@ class LocalExperimentClient:
 
         Args:
             experiment_id: Experiment UUID.
-            plot_file_name: Plot file name.
+            plot_name: Plot file name.
 
         Raises:
             RequestsApiError: If the figure is not found
@@ -621,7 +623,7 @@ class LocalExperimentClient:
         exp_id = data_dict.get("experiment_uuid")
         if exp_id is None:
             raise RequestsApiError(
-                f"Cannot create analysis result without experiment id"
+                "Cannot create analysis result without experiment id"
             )
         exp = self._experiments.loc[self._experiments.uuid == exp_id]
         if exp.empty:
@@ -660,6 +662,37 @@ class LocalExperimentClient:
         result = self._results.loc[self._results.uuid == result_id]
         return self.serialize(result)
 
+    def bulk_analysis_result_update(self, new_data: str) -> Dict:
+        """Bulk update analysis results.
+
+        Args:
+            new_data: New analysis result data array.
+
+        Returns:
+            Analysis result data.
+
+        Raises:
+            IBMExperimentEntryNotFound: If at least one analysis result is not found
+            IBMApiError: If the input is not given in the expected format
+        """
+
+        # naive implementation, can be optimized if needed
+        new_data_dict = json.loads(new_data)
+        # expected format is {"analysis_results": [...]}
+        if "analysis_results" not in new_data_dict or not isinstance(
+            new_data_dict["analysis_results"], list
+        ):
+            raise IBMApiError(
+                'Data not given in the correct bulk update format, pass {"analysis_results": [...]}'
+            )
+        response = {"analysis_results": []}
+        for new_analysis_result in new_data_dict["analysis_results"]:
+            result = self.analysis_result_update(
+                new_analysis_result["uuid"], json.dumps(new_analysis_result)
+            )
+            response["analysis_results"].append(result)
+        return response
+
     def analysis_result_delete(self, result_id: str) -> Dict:
         """Delete an analysis result.
 
