@@ -20,14 +20,19 @@ from typing import Optional, List, Dict, Union, Tuple, Any, Type
 from datetime import datetime
 from collections import defaultdict
 import requests
+import numpy as np
+from pandas import DataFrame
+import pandas as pd
 from .constants import (
     ExperimentShareLevel,
     ResultQuality,
     RESULT_QUALITY_FROM_API,
     RESULT_QUALITY_TO_API,
+    RESULT_QUALITY_FROM_DATAFRAME,
+    RESULT_QUALITY_TO_DATAFRAME,
     DEFAULT_BASE_URL,
 )
-from .utils import map_api_error, local_to_utc_str, utc_to_local
+from .utils import map_api_error, local_to_utc_str, utc_to_local, ThreadSaveHandler
 from .device_component import DeviceComponent
 from .experiment_dataclasses import ExperimentData, AnalysisResultData
 from ..client.experiment import ExperimentClient
@@ -79,7 +84,7 @@ class IBMExperimentService:
     """
 
     _default_preferences = {"auto_save": False}
-    _default_options = {"prompt_for_delete": True}
+    _default_options = {"prompt_for_delete": True, "requests_timeout": 100}
     _DEFAULT_LOCAL_DB_DIR = os.path.join(os.path.expanduser("~"), ".qiskit", "resultdb")
     _AUTHENTICATION_CMD = "/users/loginWithToken"
     _USER_DATA_CMD = "/users/me"
@@ -89,9 +94,10 @@ class IBMExperimentService:
         token: Optional[str] = None,
         url: Optional[str] = None,
         name: Optional[str] = None,
+        hgp: Optional[str] = None,
         proxies: Optional[dict] = None,
-        verify: Optional[bool] = None,
-        local: Optional[bool] = None,
+        verify: Optional[bool] = True,
+        local: Optional[bool] = False,
         local_save: Optional[bool] = True,
         **kwargs,
     ) -> None:
@@ -104,6 +110,9 @@ class IBMExperimentService:
             local_save: If using a local client, whether to enable save to disk or not.
         """
         super().__init__()
+        self.options = self._default_options
+        self.hgp = hgp
+        self.set_option(**kwargs)
         if url is None:
             url = DEFAULT_BASE_URL
         self._account = self._discover_account(
@@ -135,8 +144,6 @@ class IBMExperimentService:
             self._api_client = LocalExperimentClient(
                 main_dir=self._DEFAULT_LOCAL_DB_DIR, local_save=local_save
             )
-        self.options = self._default_options
-        self.set_option(**kwargs)
 
     @property
     def local(self):
@@ -155,18 +162,21 @@ class IBMExperimentService:
         if api_token is None:
             try:
                 api_token = self._account.token
-            except RuntimeError:
-                raise IBMApiError("No API token; cannot connect to service")
+            except RuntimeError as err:
+                raise IBMApiError("No API token; cannot connect to service") from err
         headers = {"accept": "application/json", "Content-Type": "application/json"}
         data = {"apiToken": api_token}
         url = self._account.url + self._AUTHENTICATION_CMD
-        response = requests.post(url=url, json=data, headers=headers)
+        response = requests.post(
+            url=url,
+            json=data,
+            headers=headers,
+            timeout=self.options["requests_timeout"],
+        )
         access_token = response.json().get("id", None)
         if access_token is None:
             raise IBMApiError(
-                "Did not receive access token (request returned {})".format(
-                    response.json()
-                )
+                f"Did not receive access token (request returned {response.json()})"
             )
         self._access_token = access_token
         return access_token
@@ -180,15 +190,15 @@ class IBMExperimentService:
         }
         url = self._account.url + self._USER_DATA_CMD
         try:
-            response = requests.get(url=url, headers=headers)
+            response = requests.get(
+                url=url, headers=headers, timeout=self.options["requests_timeout"]
+            )
             db_url = response.json()["urls"]["services"]["resultsDB"]
             return db_url
-        except KeyError:
+        except KeyError as err:
             raise IBMApiError(
-                "Unable to retrieve the API url for the database (request returned {})".format(
-                    response.json()
-                )
-            )
+                f"Unable to retrieve the API url for the database (request returned {response.json()})"
+            ) from err
 
     @classmethod
     def save_account(
@@ -285,30 +295,9 @@ class IBMExperimentService:
         """Create a new experiment in the database.
 
         Args:
-            experiment_type: Experiment type.
-            backend_name: Name of the backend the experiment ran on.
+            data: The dataclass containing the experiment's data
             provider: The provider used when running the experiment
-            metadata: Experiment metadata.
-            experiment_id: Experiment ID. It must be in the ``uuid4`` format.
-                One will be generated if not supplied.
-            parent_id: The experiment ID of the parent experiment.
-                The parent experiment must exist, must be on the same backend as the child,
-                and an experiment cannot be its own parent.
-            job_ids: IDs of experiment jobs.
-            tags: Tags to be associated with the experiment.
-            notes: Freeform notes about the experiment.
-            share_level: The level at which the experiment is shared. This determines who can
-                view the experiment (but not update it). This defaults to "private"
-                for new experiments. Possible values include:
-
-                - private: The experiment is only visible to its owner (default)
-                - project: The experiment is shared within its project
-                - group: The experiment is shared within its group
-                - hub: The experiment is shared within its hub
-                - public: The experiment is shared publicly regardless of provider
-            start_datetime: Timestamp when the experiment started, in local time zone.
             json_encoder: Custom JSON encoder to use to encode the experiment.
-            kwargs: Additional experiment attributes that are not supported and will be ignored.
 
         Returns:
             Experiment ID.
@@ -317,13 +306,22 @@ class IBMExperimentService:
             IBMExperimentEntryExists: If the experiment already exits.
             IBMApiError: If the request to the server failed.
         """
-        # pylint: disable=arguments-differ
+        if self.hgp is not None:
+            try:
+                data.hub, data.group, data.project = self.hgp.split("/")
+            except RuntimeError:
+                pass
 
         if provider is not None:
             # attempt to get hub/group/project data from the provider
-            data.hub = provider.credentials.hub
-            data.group = provider.credentials.group
-            data.project = provider.credentials.project
+            # old IBMQ style
+            if hasattr(provider, "credentials"):
+                data.hub = provider.credentials.hub
+                data.group = provider.credentials.group
+                data.project = provider.credentials.project
+            # new IBMProvider style
+            if hasattr(provider, "_hgps"):
+                data.hub, data.group, data.project = list(provider._hgps)[0].split("/")
 
         api_data = self._experiment_data_to_api(data)
 
@@ -351,24 +349,8 @@ class IBMExperimentService:
         """Update an existing experiment.
 
         Args:
-            experiment_id: Experiment ID.
-            metadata: Experiment metadata.
-            job_ids: IDs of experiment jobs.
-            notes: Freeform notes about the experiment.
-            tags: Tags to be associated with the experiment.
-            share_level: The level at which the experiment is shared. This determines who can
-                view the experiment (but not update it). This defaults to "private"
-                for new experiments. Possible values include:
-
-                - private: The experiment is only visible to its owner (default)
-                - project: The experiment is shared within its project
-                - group: The experiment is shared within its group
-                - hub: The experiment is shared within its hub
-                - public: The experiment is shared publicly regardless of provider
-
-            end_datetime: Timestamp for when the experiment ended, in local time.
+            data: The dataclass containing the experiment's data
             json_encoder: Custom JSON encoder to use to encode the experiment.
-            kwargs: Additional experiment attributes that are not supported and will be ignored.
 
         Raises:
             IBMExperimentEntryNotFound: If the experiment does not exist.
@@ -383,6 +365,7 @@ class IBMExperimentService:
             "hub_id",
             "project_id",
             "type",
+            "start_time",
         ]
         for field_name in unused_fields:
             if field_name in api_data:
@@ -416,15 +399,7 @@ class IBMExperimentService:
         """Convert experiment data to API request data.
 
         Args:
-            metadata: Experiment metadata.
-            experiment_id: Experiment ID.
-            parent_id: Parent experiment ID
-            job_ids: IDs of experiment jobs.
-            tags: Tags to be associated with the experiment.
-            notes: Freeform notes about the experiment.
-            share_level: The level at which the experiment is shared.
-            start_dt: Experiment start time.
-            end_dt: Experiment end time.
+            data: The dataclass containing the experiment's data
 
         Returns:
             API request data.
@@ -576,13 +551,13 @@ class IBMExperimentService:
             parent_id: Filter experiments by this parent experiment ID.
             sort_by: Specifies how the output should be sorted. This can be a single sorting
                 option or a list of options. Each option should contain a sort key
-                and a direction, separated by a semicolon. Valid sort keys are
-                "start_datetime" and "experiment_type".
+                and a direction, separated by a colon. Valid sort keys are
+                "start_time" and "experiment_type".
                 Valid directions are "asc" for ascending or "desc" for descending.
-                For example, ``sort_by=["experiment_type:asc", "start_datetime:desc"]`` will
+                For example, ``sort_by=["experiment_type:asc", "start_time:desc"]`` will
                 return an output list that is first sorted by experiment type in
                 ascending order, then by start datetime by descending order.
-                By default, experiments are sorted by ``start_datetime``
+                By default, experiments are sorted by ``start_time``
                 descending and ``experiment_id`` ascending.
             **filters: Additional filtering keywords that are not supported and will be ignored.
 
@@ -595,6 +570,7 @@ class IBMExperimentService:
             IBMApiError: If the request to the server failed.
         """
         # pylint: disable=arguments-differ
+        # pylint: disable=missing-param-doc
         if filters:
             logger.info(
                 "Keywords %s are not supported by IBM Quantum experiment service "
@@ -618,10 +594,10 @@ class IBMExperimentService:
 
         start_time_filters = []
         if start_datetime_after is not None:
-            st_filter = "ge:{}".format(local_to_utc_str(start_datetime_after))
+            st_filter = f"ge:{local_to_utc_str(start_datetime_after)}"
             start_time_filters.append(st_filter)
         if start_datetime_before is not None:
-            st_filter = "le:{}".format(local_to_utc_str(start_datetime_before))
+            st_filter = f"le:{local_to_utc_str(start_datetime_before)}"
             start_time_filters.append(st_filter)
 
         if exclude_public and public_only:
@@ -634,7 +610,11 @@ class IBMExperimentService:
             tags=tags,
             tags_operator=tags_operator,
             sort_by=sort_by,
-            sort_map={"start_datetime": "start_time", "experiment_type": "type"},
+            sort_map={
+                "start_datetime": "start_time",
+                "start_time": "start_time",
+                "experiment_type": "type",
+            },
             device_components=device_components,
             device_components_operator=device_components_operator,
             item_type=experiment_type,
@@ -644,7 +624,7 @@ class IBMExperimentService:
         experiments = []
         marker = None
         while limit is None or limit > 0:
-            with map_api_error(f"Request failed."):
+            with map_api_error("Request failed."):
                 response = self._api_client.experiments(
                     limit=limit,
                     marker=marker,
@@ -795,7 +775,7 @@ class IBMExperimentService:
 
         Args:
             data: The data to save. Note that the following fields will be ignored:
-            'uuid', 'experiment_uuid', 'device_components', 'type'
+            'experiment_uuid', 'device_components', 'type'
             json_encoder: Custom JSON encoder to use to encode the analysis result.
 
         Raises:
@@ -830,6 +810,35 @@ class IBMExperimentService:
             max_attempts,
         )
 
+    def bulk_update_analysis_result(
+        self,
+        data: List[AnalysisResultData],
+        json_encoder: Type[json.JSONEncoder] = json.JSONEncoder,
+    ) -> None:
+        """Bulk updates existing analysis results.
+
+        Args:
+            data: An array of the analysis data to save. Note that the following fields will be ignored:
+            ''experiment_uuid', 'device_components', 'type'
+            json_encoder: Custom JSON encoder to use to encode the analysis result.
+
+        Raises:
+            IBMApiError: If the request to the server failed.
+        """
+
+        unused_fields = ["experiment_uuid", "device_components", "type"]
+        request_list = {"analysis_results": []}
+        for analysis_result in data:
+            request = self._analysis_result_to_api(analysis_result)
+            for field_name in unused_fields:
+                if field_name in request:
+                    del request[field_name]
+            request_list["analysis_results"].append(request)
+        with map_api_error("Bulk analysis result update failed."):
+            self._api_client.bulk_analysis_result_update(
+                json.dumps(request_list, cls=json_encoder)
+            )
+
     def _confirm_delete(self, msg: str) -> bool:
         """Confirms a delete command; if the options indicate a prompt should be
         dislayed, display one and verify the user input"""
@@ -840,6 +849,47 @@ class IBMExperimentService:
             return False
         return True
 
+    def create_analysis_results(
+        self,
+        data: Union[List[AnalysisResultData], DataFrame],
+        blocking: bool = True,
+        max_workers: int = 100,
+        json_encoder: Type[json.JSONEncoder] = json.JSONEncoder,
+    ):
+        """Create multiple analysis results in the database using asynchronous calls.
+
+        If you choose `blocking==True`, the method will run until all the save threads terminated.
+        To improve running time, multithreading is used.
+
+        If `blocking==False` it is up to the user to verify all the threads finished;
+        `block_for_save()` can be called to ensure all threads finish.
+        `save_status()` returns the information on the status of the threads.
+
+        Args:
+            data: The data to save, either as a list of `AnalysisResultData` or as a pandas `DataFrame`.
+            blocking: Whether to wait for all the save threads to finish before returning control
+            max_workers: Maximum number of worker threads to write to the server.
+            json_encoder: Custom JSON encoder to use to encode the analysis result.
+
+        Raises:
+            IBMExperimentEntryExists: If the analysis result already exits.
+            IBMApiError: If the request to the server failed.
+        """
+        if isinstance(data, DataFrame):
+            data = self.dataframe_to_analysis_result_list(data)
+        handler = ThreadSaveHandler(
+            data,
+            self.create_or_update_analysis_result,
+            max_workers,
+            json_encoder=json_encoder,
+            create=True,
+            max_attempts=3,
+        )
+        if blocking:
+            handler.block_for_save()
+            return handler.save_status()
+        return handler
+
     def _analysis_result_to_api(self, data: AnalysisResultData) -> Dict:
         """Convert analysis result fields to server format.
 
@@ -993,6 +1043,8 @@ class IBMExperimentService:
             IBMApiError: If the request to the server failed.
         """
         # pylint: disable=arguments-differ
+        # pylint: disable=missing-param-doc
+
         if filters:
             logger.info(
                 "Keywords %s are not supported by IBM Quantum experiment service "
@@ -1009,10 +1061,10 @@ class IBMExperimentService:
 
         created_at_filters = []
         if creation_datetime_after:
-            ca_filter = "ge:{}".format(local_to_utc_str(creation_datetime_after))
+            ca_filter = f"ge:{local_to_utc_str(creation_datetime_after)}"
             created_at_filters.append(ca_filter)
         if creation_datetime_before:
-            ca_filter = "le:{}".format(local_to_utc_str(creation_datetime_before))
+            ca_filter = f"le:{local_to_utc_str(creation_datetime_before)}"
             created_at_filters.append(ca_filter)
 
         converted = self._filtering_to_api(
@@ -1122,8 +1174,8 @@ class IBMExperimentService:
                 tags_filter = "contains:" + ",".join(tags)
             else:
                 raise ValueError(
-                    "{} is not a valid `tags_operator`. Valid values are "
-                    '"AND" and "OR".'.format(tags_operator)
+                    f"{tags_operator} is not a valid `tags_operator`. Valid values are "
+                    '"AND" and "OR".'
                 )
 
         sort_list = []
@@ -1221,6 +1273,88 @@ class IBMExperimentService:
         }
         return out_dict
 
+    @staticmethod
+    def _dataframe_to_analysis_result(
+        raw_data: Dict,
+    ) -> AnalysisResultData:
+        """Map dataframe dictionary to an analysis result.
+
+        Args:
+            raw_data: Dataframe dictionary data
+
+        Returns:
+            Converted analysis result data.
+        """
+
+        # raw data might contain iterated data structures unknown to us, so deep copy to prevent problems
+        raw_data = copy.deepcopy(raw_data)
+
+        data_field_map = {
+            "name": "result_type",
+            "components": "device_components",
+            "_result_id": "result_id",
+            "_experiment_id": "experiment_id",
+            "_tags": "tags",
+            "chisq": "chisq",
+            "created_time": "creation_datetime",
+            "backend": "backend_name",
+        }
+        analysis_result_data = {}
+        for src_key, dest_key in data_field_map.items():
+            if src_key in raw_data:
+                analysis_result_data[dest_key] = raw_data[src_key]
+
+        # extra data is stored in the 'result_data' field
+        result_data_field_map = {
+            "value": "_value",
+            "_source": "_source",
+            "_extra": "_extra",
+            "experiment": "_experiment",
+        }
+        result_data = {}
+        for src_key, dest_key in result_data_field_map.items():
+            if src_key in raw_data:
+                result_data[dest_key] = raw_data[src_key]
+        analysis_result_data["result_data"] = result_data
+
+        # values which require specific conversions
+        analysis_result_data["quality"] = RESULT_QUALITY_FROM_DATAFRAME[
+            raw_data.get("quality", "unknown")
+        ]
+        return AnalysisResultData(**analysis_result_data)
+
+    @staticmethod
+    def _analysis_result_to_dataframe(
+        raw_data: AnalysisResultData,
+    ) -> Dict:
+        """Map analysis result to a dataframe dictionary.
+
+        Args:
+            raw_data: Analysis result data
+
+        Returns:
+            Converted analysis result data dictionary.
+        """
+
+        # raw data might contain iterated data structures unknown to us, so deep copy to prevent problems
+        raw_data = copy.deepcopy(raw_data)
+        analysis_result_data = {
+            "name": raw_data.result_type,
+            "components": raw_data.device_components,
+            "_result_id": raw_data.result_id,
+            "_experiment_id": raw_data.experiment_id,
+            "_tags": raw_data.tags,
+            "chisq": raw_data.chisq,
+            "created_time": raw_data.creation_datetime,
+            "value": raw_data.result_data.get("_value", None),
+            "_source": raw_data.result_data.get("_source", None),
+            "_extra": raw_data.result_data.get("_extra", None),
+            "experiment": raw_data.result_data.get("_experiment", None),
+            "quality": RESULT_QUALITY_TO_DATAFRAME[raw_data.quality],
+            "backend": raw_data.backend_name,
+        }
+        return analysis_result_data
+
     def delete_analysis_result(self, result_id: str) -> None:
         """Delete an analysis result.
 
@@ -1273,7 +1407,7 @@ class IBMExperimentService:
             if isinstance(figure, str):
                 figure_name = figure
             else:
-                figure_name = "figure_{}.svg".format(datetime.now().isoformat())
+                figure_name = f"figure_{datetime.now().isoformat()}.svg"
 
         # currently the resultdb enforces files to end with .svg
         if not figure_name.endswith(".svg"):
@@ -1292,6 +1426,52 @@ class IBMExperimentService:
             return None
         return figure_name, len(figure)
 
+    def create_figures(
+        self,
+        experiment_id: str,
+        figure_list: List[Tuple[Union[str, bytes], str]],
+        blocking: bool = True,
+        max_workers: int = 100,
+    ):
+        """Create multiple figures in the database using asynchronous calls.
+
+        If you choose `blocking==True`, the method will run until all the save threads terminated.
+        To improve running time, multithreading is used.
+
+        If `blocking==False` it is up to the user to verify all the threads finished;
+        `block_for_save()` can be called to ensure all threads finish.
+        `save_status()` returns the information on the status of the threads.
+
+        Args:
+            experiment_id: ID of the experiment this figure is for.
+            figure_list: A list of the figures to save.
+                Every figure is given by a tuple of the form (figure, name)
+                where `figure` can be the actual figure or its filename
+                and `name` is the name given to the figure in the db.
+                If ``None``, the figure file name, if given, or a generated name is used.
+            blocking: Whether to wait for all the save threads to finish before returning control
+            max_workers: Maximum number of worker threads to write to the server.
+
+        Raises:
+            IBMExperimentEntryExists: If the figure already exits.
+            IBMApiError: If the request to the server failed.
+        """
+        figure_params = [
+            (experiment_id, figure, figure_name)
+            for (figure, figure_name) in figure_list
+        ]
+        handler = ThreadSaveHandler(
+            figure_params,
+            self.create_or_update_figure,
+            max_workers,
+            create=True,
+            max_attempts=3,
+        )
+        if blocking:
+            handler.block_for_save()
+            return handler.save_status()
+        return handler
+
     def update_figure(
         self,
         experiment_id: str,
@@ -1316,7 +1496,7 @@ class IBMExperimentService:
             if isinstance(figure, str):
                 figure_name = figure
             else:
-                figure_name = "figure_{}.svg".format(datetime.now().isoformat())
+                figure_name = f"figure_{datetime.now().isoformat()}.svg"
 
         # currently the resultdb enforces files to end with .svg
         if not figure_name.endswith(".svg"):
@@ -1512,7 +1692,7 @@ class IBMExperimentService:
         # currently the resultdb enforces files to end with .json or .yaml
         if not file_name.endswith(".json"):
             file_name += ".json"
-        if isinstance(file_data, Dict):
+        if isinstance(file_data, dict):
             file_data = json.dumps(file_data)
         self._api_client.experiment_file_upload(experiment_id, file_name, file_data)
 
@@ -1532,7 +1712,7 @@ class IBMExperimentService:
         file_data = self._api_client.experiment_file_download(experiment_id, file_name)
         return file_data
 
-    def experiment_has_file(self, exp_id: str, filename: str) -> bool:
+    def experiment_has_file(self, experiment_id: str, file_name: str) -> bool:
         """Checks whether a specific expriment has a specific file
         Args:
             experiment_id: The experiment the data file belongs to
@@ -1540,9 +1720,9 @@ class IBMExperimentService:
         Returns:
             True if the file exists for the specified experiment
         """
-        files = self.files(exp_id)["files"]
+        files = self.files(experiment_id)["files"]
         for file_data in files:
-            if file_data["Key"] == filename:
+            if file_data["Key"] == file_name:
                 return True
         return False
 
@@ -1566,8 +1746,6 @@ class IBMExperimentService:
 
         Args:
             name: Name of the saved account to delete.
-            auth: Authentication type of the default account to delete.
-                Ignored if account name is provided.
 
         Returns:
             True if the account was deleted.
@@ -1575,3 +1753,28 @@ class IBMExperimentService:
         """
 
         return AccountManager.delete(name=name)
+
+    @staticmethod
+    def dataframe_to_analysis_result_list(df: DataFrame) -> List[AnalysisResultData]:
+        """Converts an analysis result dataframe to a list"""
+        results = []
+        data_dict = df.replace({np.nan: None}).to_dict("records")
+        for result in data_dict:
+            results.append(IBMExperimentService._dataframe_to_analysis_result(result))
+        return results
+
+    @staticmethod
+    def analysis_result_list_to_dataframe(
+        result_list: List[AnalysisResultData],
+    ) -> DataFrame:
+        """Converts a list of analysis results to a pandas dataframe"""
+        if len(result_list) == 0:
+            return pd.DataFrame()
+        result_dicts = [
+            IBMExperimentService._analysis_result_to_dataframe(result)
+            for result in result_list
+        ]
+        columns = result_dicts[0].keys()
+        pandas_dict = {key: [result[key] for result in result_dicts] for key in columns}
+        df = DataFrame.from_dict(pandas_dict)
+        return df
