@@ -16,17 +16,17 @@ import os
 import uuid
 import unittest
 import json
+import re
 from unittest import mock, SkipTest, skipIf
 from datetime import datetime, timedelta
 from typing import Optional, Dict, Any
-import re
 from test.service.ibm_test_case import IBMTestCase
 from test.utils.utils import ExperimentEncoder, ExperimentDecoder
 import numpy as np
 from dateutil import tz
-from qiskit.providers.ibmq import IBMQFactory, least_busy
+from qiskit_ibm_provider import IBMProvider
 from qiskit_ibm_experiment.service import ResultQuality, ExperimentShareLevel
-from qiskit_ibm_experiment import IBMExperimentEntryNotFound
+from qiskit_ibm_experiment import IBMExperimentEntryNotFound, IBMApiError
 from qiskit_ibm_experiment import IBMExperimentService
 from qiskit_ibm_experiment import ExperimentData, AnalysisResultData
 
@@ -43,12 +43,11 @@ class TestExperimentServerIntegration(IBMTestCase):
         super().setUpClass()
         cls.default_exp_type = "qiskit_test"
         try:
-            cls._setup_provider()
             cls._setup_service()
-            cls.device_components = cls.service.device_components(cls.backend.name())
+            cls._setup_provider()
         except Exception as err:
             cls.log.info("Error while setting the service/provider: %s", err)
-            raise SkipTest("Not authorized to use experiment service.")
+            raise SkipTest("Not authorized to use experiment service.") from err
 
     @classmethod
     def _setup_service(cls):
@@ -56,16 +55,22 @@ class TestExperimentServerIntegration(IBMTestCase):
         cls.service = IBMExperimentService(
             token=os.getenv("QISKIT_IBM_STAGING_API_TOKEN"),
             url=os.getenv("QISKIT_IBM_STAGING_API_URL"),
+            hgp=os.getenv("QISKIT_IBM_STAGING_HGP"),
         )
 
     @classmethod
     def _setup_provider(cls):
         """Get the provider for the class."""
-        cls.provider = IBMQFactory().enable_account(
+        cls.provider = IBMProvider(
             token=os.getenv("QISKIT_IBM_STAGING_API_TOKEN"),
             url=os.getenv("QISKIT_IBM_STAGING_API_URL"),
+            instance=os.getenv("QISKIT_IBM_STAGING_HGP"),
         )
-        cls.backend = least_busy(cls.provider.backends())
+        cls.backend = cls.provider.get_backend(os.getenv("QISKIT_IBM_STAGING_BACKEND"))
+        try:
+            cls.device_components = cls.service.device_components(cls.backend.name)
+        except IBMApiError:
+            cls.device_components = None
 
     @classmethod
     def get_experiments(cls, **kwargs):
@@ -99,7 +104,7 @@ class TestExperimentServerIntegration(IBMTestCase):
 
         found = False
         for exp in experiments:
-            self.assertTrue(exp.experiment_id, "{} does not have an ID!".format(exp))
+            self.assertTrue(exp.experiment_id, f"{exp} does not have an ID!")
             for dt_attr in [
                 "start_datetime",
                 "creation_datetime",
@@ -115,18 +120,16 @@ class TestExperimentServerIntegration(IBMTestCase):
     def test_experiments_with_backend(self):
         """Test retrieving all experiments for a specific backend."""
         exp_id = self._create_experiment()
-        backend_experiments = self.get_experiments(backend_name=self.backend.name())
+        backend_experiments = self.get_experiments(backend_name=self.backend.name)
 
         found = False
         for exp in backend_experiments:
-            self.assertEqual(self.backend.name(), exp.backend)
+            self.assertEqual(self.backend.name, exp.backend)
             if exp.experiment_id == exp_id:
                 found = True
         self.assertTrue(
             found,
-            "Experiment {} not found when filter by backend name {}.".format(
-                exp_id, self.backend.name()
-            ),
+            f"Experiment {exp_id} not found when filter by backend name {self.backend.name}.",
         )
 
     def test_experiments_with_type(self):
@@ -141,8 +144,7 @@ class TestExperimentServerIntegration(IBMTestCase):
             if exp.experiment_id == exp_id:
                 found = True
         self.assertTrue(
-            found,
-            "Experiment {} not found when filter by type {}.".format(exp_id, exp_type),
+            found, f"Experiment {exp_id} not found when filter by type {exp_type}."
         )
 
     def test_experiments_with_parent_id(self):
@@ -157,10 +159,7 @@ class TestExperimentServerIntegration(IBMTestCase):
             if exp.experiment_id == child_id:
                 found = True
         self.assertTrue(
-            found,
-            "Experiment {} not found when filter by type {}.".format(
-                child_id, parent_id
-            ),
+            found, f"Experiment {child_id} not found when filter by type {parent_id}."
         )
 
     def test_experiments_with_type_operator(self):
@@ -203,7 +202,6 @@ class TestExperimentServerIntegration(IBMTestCase):
 
         before_start = ref_start_dt - timedelta(hours=1)
         after_start = ref_start_dt + timedelta(hours=1)
-        exp = self.service.experiments(start_datetime_after=before_start)
 
         sub_tests = [
             (before_start, None, True, "before start, None"),
@@ -232,10 +230,8 @@ class TestExperimentServerIntegration(IBMTestCase):
                 self.assertEqual(
                     found,
                     expected,
-                    "Experiment {} (not)found unexpectedly when filter using "
-                    "start_dt={}, end_dt={}. Found={}".format(
-                        exp_id, start_dt, end_dt, found
-                    ),
+                    f"Experiment {exp_id} (not)found unexpectedly when filter using "
+                    f"start_dt={start_dt}, end_dt={end_dt}. Found={found}",
                 )
 
     def test_experiments_with_tags(self):
@@ -257,9 +253,7 @@ class TestExperimentServerIntegration(IBMTestCase):
                 experiments = self.get_experiments(tags=tags, tags_operator=operator)
                 ref_expr_found = False
                 for expr in experiments:
-                    msg = "Tags {} not fond in experiment tags {}".format(
-                        tags, expr.tags
-                    )
+                    msg = f"Tags {tags} not fond in experiment tags {expr.tags}"
                     if operator == "AND":
                         self.assertTrue(all(f_tag in expr.tags for f_tag in tags), msg)
                     else:
@@ -268,20 +262,17 @@ class TestExperimentServerIntegration(IBMTestCase):
                         ref_expr_found = True
                 self.assertTrue(
                     ref_expr_found == found,
-                    "Experiment tags {} unexpectedly (not)found. Found={}".format(
-                        ref_tags, found
-                    ),
+                    f"Experiment tags {ref_tags} unexpectedly (not)found. Found={found}",
                 )
 
     def test_experiments_with_hgp(self):
         """Test retrieving all experiments for a specific h/g/p."""
         exp_id = self._create_experiment()
-        credentials = self.provider.credentials
-        hgp = [credentials.hub, credentials.group, credentials.project]
+        hub, group, project = list(self.provider._hgps)[0].split("/")
         sub_tests = [
-            {"hub": hgp[0]},
-            {"hub": hgp[0], "group": hgp[1]},
-            {"hub": hgp[0], "group": hgp[1], "project": hgp[2]},
+            {"hub": hub},
+            {"hub": hub, "group": group},
+            {"hub": hub, "group": group, "project": project},
         ]
 
         for hgp_kwargs in sub_tests:
@@ -327,20 +318,18 @@ class TestExperimentServerIntegration(IBMTestCase):
             self.assertNotEqual(
                 experiment.share_level,
                 ExperimentShareLevel.PUBLIC.value,
-                "Public experiment should not be returned with exclude_public filter: %s"
-                % experiment,
+                f"Public experiment should not be returned with exclude_public filter: {experiment}",
             )
             non_public_experiment_uuids.append(experiment.experiment_id)
         self.assertIn(
             private_exp_id,
             non_public_experiment_uuids,
-            "Non-public experiment not returned with exclude_public filter: %s"
-            % private_exp_id,
+            f"Non-public experiment not returned with exclude_public filter:{private_exp_id}",
         )
         self.assertNotIn(
             public_exp_id,
             non_public_experiment_uuids,
-            "Public experiment returned with exclude_public filter: %s" % public_exp_id,
+            f"Public experiment returned with exclude_public filter: {public_exp_id}",
         )
 
     def test_experiments_with_public_only(self):
@@ -358,21 +347,18 @@ class TestExperimentServerIntegration(IBMTestCase):
             self.assertEqual(
                 experiment.share_level,
                 ExperimentShareLevel.PUBLIC.value,
-                "Only public experiments should be returned with public_only filter: %s"
-                % experiment,
+                f"Only public experiments should be returned with public_only filter: {experiment}",
             )
             public_experiment_uuids.append(experiment.experiment_id)
         self.assertIn(
             public_exp_id,
             public_experiment_uuids,
-            "Public experiment not returned with public_only filter: %s"
-            % public_exp_id,
+            f"Public experiment not returned with public_only filter: {public_exp_id}",
         )
         self.assertNotIn(
             private_exp_id,
             public_experiment_uuids,
-            "Non-public experiment returned with public_only filter: %s"
-            % private_exp_id,
+            f"Non-public experiment returned with public_only filter: {private_exp_id}",
         )
 
     def test_experiments_with_public_filters_error(self):
@@ -399,14 +385,14 @@ class TestExperimentServerIntegration(IBMTestCase):
             self.assertNotEqual(
                 experiment["owner"],
                 exp_owner,  # pylint: disable=no-member
-                "My experiment should not be returned with exclude_mine filter: %s"
-                % experiment["experiment_id"],
+                f"My experiment should not be returned with "
+                f"exclude_mine filter: {experiment['experiment_id']}",
             )
             not_mine_experiment_uuids.append(experiment["experiment_id"])
         self.assertNotIn(
             exp_id,
             not_mine_experiment_uuids,
-            "My experiment returned with exclude_mine filter: %s" % exp_id,
+            f"My experiment returned with exclude_mine filter: {exp_id}",
         )
 
     def test_experiments_with_mine_only(self):
@@ -424,14 +410,14 @@ class TestExperimentServerIntegration(IBMTestCase):
             self.assertEqual(
                 experiment.owner,
                 exp_owner,  # pylint: disable=no-member
-                "Only my experiments should be returned with mine_only filter: %s"
-                % experiment.experiment_id,
+                f"Only my experiments should be returned with "
+                f"mine_only filter: {experiment.experiment_id}",
             )
             my_experiment_uuids.append(experiment.experiment_id)
         self.assertIn(
             exp_id,
             my_experiment_uuids,
-            "My experiment not returned with mine_only filter: %s" % exp_id,
+            f"My experiment not returned with mine_only filter: {exp_id}",
         )
 
     def test_experiments_with_owner_filters_error(self):
@@ -460,17 +446,17 @@ class TestExperimentServerIntegration(IBMTestCase):
         tags = [str(uuid.uuid4())]
         exp1 = self._create_experiment(
             tags=tags,
-            experiment_type="{}1".format(self.default_exp_type),
+            experiment_type=f"{self.default_exp_type}1",
             start_datetime=datetime.now() - timedelta(hours=1),
         )
         exp2 = self._create_experiment(
             tags=tags,
-            experiment_type="{}2".format(self.default_exp_type),
+            experiment_type=f"{self.default_exp_type}2",
             start_datetime=datetime.now(),
         )
         exp3 = self._create_experiment(
             tags=tags,
-            experiment_type="{}1".format(self.default_exp_type),
+            experiment_type=f"{self.default_exp_type}1",
             start_datetime=datetime.now() - timedelta(hours=2),
         )
 
@@ -558,9 +544,7 @@ class TestExperimentServerIntegration(IBMTestCase):
         rexp = self.service.experiment(exp_id)
         self.assertEqual(exp_id, rexp.experiment_id)
         for attr in ["hub", "group", "project", "owner", "share_level"]:
-            self.assertIsNotNone(
-                getattr(rexp, attr), "{} does not have a {}".format(rexp, attr)
-            )
+            self.assertIsNotNone(getattr(rexp, attr), f"{rexp} does not have a {attr}")
 
     def test_upload_experiment(self):
         """Test uploading an experiment."""
@@ -568,7 +552,7 @@ class TestExperimentServerIntegration(IBMTestCase):
         new_exp_id = self.service.create_experiment(
             ExperimentData(
                 experiment_type="qiskit_test",
-                backend=self.backend.name(),
+                backend=self.backend.name,
                 metadata={"foo": "bar"},
                 experiment_id=exp_id,
                 job_ids=["job1", "job2"],
@@ -583,14 +567,12 @@ class TestExperimentServerIntegration(IBMTestCase):
         self.assertEqual(exp_id, new_exp_id)
         new_exp = self.service.experiment(new_exp_id)
 
-        credentials = self.provider.credentials
-        self.assertEqual(credentials.hub, new_exp.hub)  # pylint: disable=no-member
-        self.assertEqual(credentials.group, new_exp.group)  # pylint: disable=no-member
-        self.assertEqual(
-            credentials.project, new_exp.project
-        )  # pylint: disable=no-member
+        hub, group, project = list(self.provider._hgps)[0].split("/")
+        self.assertEqual(hub, new_exp.hub)  # pylint: disable=no-member
+        self.assertEqual(group, new_exp.group)  # pylint: disable=no-member
+        self.assertEqual(project, new_exp.project)  # pylint: disable=no-member
         self.assertEqual("qiskit_test", new_exp.experiment_type)
-        self.assertEqual(self.backend.name(), new_exp.backend)
+        self.assertEqual(self.backend.name, new_exp.backend)
         self.assertEqual({"foo": "bar"}, new_exp.metadata)
         self.assertEqual(["job1", "job2"], new_exp.job_ids)
         self.assertEqual(["qiskit_test"], new_exp.tags)
@@ -637,7 +619,7 @@ class TestExperimentServerIntegration(IBMTestCase):
     def test_create_or_update_experiment(self):
         """Test updating an experiment."""
         new_exp_id = self.service.create_or_update_experiment(
-            ExperimentData(experiment_type="qiskit_test", backend=self.backend.name()),
+            ExperimentData(experiment_type="qiskit_test", backend=self.backend.name),
             provider=self.provider,
         )
 
@@ -706,6 +688,121 @@ class TestExperimentServerIntegration(IBMTestCase):
         self.assertEqual(result_id, rresult.result_id)
         self.assertEqual(chisq, rresult.chisq)
 
+    def test_upload_multiple_analysis_results(self):
+        """Test uploading multiple analysis results."""
+        exp_id = self._create_experiment()
+        num_results = 10
+        results = []
+        for i in range(num_results):
+            fit = dict(value=i + 5, variance=2)
+            chisq = 1.3253
+            result = AnalysisResultData(
+                experiment_id=exp_id,
+                result_type=f"{i}_qiskit_test",
+                result_data=fit,
+                device_components=self.device_components,
+                tags=["qiskit_test"],
+                quality=ResultQuality.GOOD,
+                verified=True,
+                chisq=chisq,
+            )
+            results.append(result)
+        self.service.create_analysis_results(results, blocking=True)
+        rresults = self.service.analysis_results(
+            experiment_id=exp_id, limit=num_results
+        )
+        self.assertEqual(len(rresults), num_results)
+        for rresult in rresults:
+            if rresult.result_type == "qiskit_test":
+                print(rresult)
+            i = int(re.match(r"(\d+)_", rresult.result_type)[1])
+            fit = dict(value=i + 5, variance=2)
+            self.assertEqual(exp_id, rresult.experiment_id)
+            self.assertEqual(f"{i}_qiskit_test", rresult.result_type)
+            self.assertEqual(fit, rresult.result_data)
+            self.assertEqual(
+                self.device_components,
+                [str(comp) for comp in rresult.device_components],
+            )
+            self.assertEqual(["qiskit_test"], rresult.tags)
+            self.assertEqual(ResultQuality.GOOD, rresult.quality)
+            self.assertTrue(rresult.verified)
+            self.assertEqual(chisq, rresult.chisq)
+
+    def test_upload_multiple_analysis_results_nonblocking(self):
+        """Test uploading multiple analysis results."""
+        exp_id = self._create_experiment()
+        num_results = 100
+        results = []
+        for i in range(num_results):
+            fit = dict(value=i + 5, variance=2)
+            chisq = 1.3253
+            result = AnalysisResultData(
+                experiment_id=exp_id,
+                result_type=f"{i}_qiskit_test",
+                result_data=fit,
+                device_components=self.device_components,
+                tags=["qiskit_test"],
+                quality=ResultQuality.GOOD,
+                verified=True,
+                chisq=chisq,
+            )
+            results.append(result)
+        handler = self.service.create_analysis_results(results, blocking=False)
+        handler.block_for_save()
+        rresults = self.service.analysis_results(
+            experiment_id=exp_id, limit=num_results
+        )
+        self.assertEqual(len(rresults), num_results)
+        for rresult in rresults:
+            if rresult.result_type == "qiskit_test":
+                print(rresult)
+            i = int(re.match(r"(\d+)_", rresult.result_type)[1])
+            fit = dict(value=i + 5, variance=2)
+            self.assertEqual(exp_id, rresult.experiment_id)
+            self.assertEqual(f"{i}_qiskit_test", rresult.result_type)
+            self.assertEqual(fit, rresult.result_data)
+            self.assertEqual(
+                self.device_components,
+                [str(comp) for comp in rresult.device_components],
+            )
+            self.assertEqual(["qiskit_test"], rresult.tags)
+            self.assertEqual(ResultQuality.GOOD, rresult.quality)
+            self.assertTrue(rresult.verified)
+            self.assertEqual(chisq, rresult.chisq)
+
+    def test_upload_multiple_analysis_results_failures(self):
+        """Test uploading multiple analysis results."""
+        exp_id = self._create_experiment()
+        fake_exp_id = f"FAKE_ID_{exp_id}"
+        num_results = 9
+        results = []
+        for i in range(num_results):
+            fit = dict(value=i + 5, variance=2)
+            chisq = 1.3253
+            result = AnalysisResultData(
+                experiment_id=exp_id if i % 2 == 0 else fake_exp_id,
+                result_type=f"{i}_qiskit_test",
+                result_data=fit,
+                device_components=self.device_components,
+                tags=["qiskit_test"],
+                quality=ResultQuality.GOOD,
+                verified=True,
+                chisq=chisq,
+            )
+            results.append(result)
+        save_status = self.service.create_analysis_results(results, blocking=True)
+        self.assertEqual(len(save_status["running"]), 0)
+        self.assertEqual(len(save_status["fail"]), num_results // 2)
+        self.assertEqual(len(save_status["done"]), num_results - (num_results // 2))
+        for result in save_status["done"]:
+            i = int(re.match(r"(\d+)_", result[0].result_type)[1])
+            self.assertEqual(i % 2, 0)
+        for result in save_status["fail"]:
+            i = int(re.match(r"(\d+)_", result["data"][0].result_type)[1])
+            self.assertEqual(i % 2, 1)
+            self.assertTrue(isinstance(result["exception"], IBMApiError))
+
     def test_update_analysis_result(self):
         """Test updating an analysis result."""
         result_id = self._create_analysis_result()
@@ -731,6 +828,37 @@ class TestExperimentServerIntegration(IBMTestCase):
         self.assertTrue(rresult.verified)
         self.assertEqual(chisq, rresult.chisq)
 
+    def test_bulk_update_analysis_result(self):
+        """Test bulk updating analysis results."""
+        num_results = 4
+        result_ids = [self._create_analysis_result() for _ in range(num_results)]
+        fits = [
+            dict(value=41.456 + i * 0.17, variance=4.051 + i * 0.53)
+            for i in range(num_results)
+        ]
+        chisqs = [1.3253 + i * 0.13 for i in range(num_results)]
+
+        new_results = [
+            AnalysisResultData(
+                result_id=result_ids[i],
+                result_data=fits[i],
+                tags=["qiskit_test"],
+                quality=ResultQuality.GOOD,
+                verified=True,
+                chisq=chisqs[i],
+            )
+            for i in range(num_results)
+        ]
+        self.service.bulk_update_analysis_result(new_results)
+        for i in range(num_results):
+            rresult = self.service.analysis_result(result_ids[i])
+            self.assertEqual(result_ids[i], rresult.result_id)
+            self.assertEqual(fits[i], rresult.result_data)
+            self.assertEqual(["qiskit_test"], rresult.tags)
+            self.assertEqual(ResultQuality.GOOD, rresult.quality)
+            self.assertTrue(rresult.verified)
+            self.assertEqual(chisqs[i], rresult.chisq)
+
     def test_create_or_update_analysis_result(self):
         """Test updating an analysis result."""
         experiment_id = self._create_experiment()
@@ -774,7 +902,7 @@ class TestExperimentServerIntegration(IBMTestCase):
         for res in results:
             self.assertIsInstance(res.verified, bool)
             self.assertIsInstance(res.result_data, dict)
-            self.assertTrue(res.result_id, "{} does not have an uuid!".format(res))
+            self.assertTrue(res.result_id, f"{res} does not have an uuid!")
             for dt_attr in ["creation_datetime", "updated_datetime"]:
                 result_datetime = getattr(res, dt_attr)
                 self.assertTrue(result_datetime is None or result_datetime.tzinfo)
@@ -879,10 +1007,8 @@ class TestExperimentServerIntegration(IBMTestCase):
                 self.assertEqual(
                     found,
                     expected,
-                    "Analysis result {} (not)found unexpectedly when filter using"
-                    "start_dt={}, end_dt={}. Found={}".format(
-                        result_id, start_dt, end_dt, found
-                    ),
+                    f"Analysis result {result_id} (not)found unexpectedly when filter using"
+                    f"start_dt={start_dt}, end_dt={end_dt}. Found={found}",
                 )
 
     def test_analysis_results_type(self):
@@ -983,7 +1109,7 @@ class TestExperimentServerIntegration(IBMTestCase):
     def test_analysis_results_backend_name(self):
         """Test filtering analysis results with backend name."""
         result_id = self._create_analysis_result()
-        results = self.service.analysis_results(backend_name=self.backend.name())
+        results = self.service.analysis_results(backend_name=self.backend.name)
         self.assertIn(result_id, [res.result_id for res in results])
 
     def test_analysis_results_verified(self):
@@ -1020,7 +1146,7 @@ class TestExperimentServerIntegration(IBMTestCase):
                 )
                 res_found = False
                 for res in results:
-                    msg = "Tags {} not fond in result tags {}".format(tags, res.tags)
+                    msg = f"Tags {tags} not fond in result tags {res.tags}"
                     if operator == "AND":
                         self.assertTrue(all(f_tag in res.tags for f_tag in tags), msg)
                     else:
@@ -1029,9 +1155,7 @@ class TestExperimentServerIntegration(IBMTestCase):
                         res_found = True
                 self.assertTrue(
                     res_found == found,
-                    "Result tags {} unexpectedly (not)found. Found={}".format(
-                        ref_tags, found
-                    ),
+                    f"Result tags {ref_tags} unexpectedly (not)found. Found={found}",
                 )
 
     def test_analysis_results_with_limit(self):
@@ -1052,7 +1176,7 @@ class TestExperimentServerIntegration(IBMTestCase):
         """Test retrieving analysis results with sort_by."""
         tags = [str(uuid.uuid4())]
         backend, components = self._find_backend_device_components(3)
-        backend_name = backend or self.backend.name()
+        backend_name = backend or self.backend.name
         device_components = components or self.device_components
         if len(device_components) < 3:
             device_components = [None] * 3  # Skip testing device components.
@@ -1172,7 +1296,7 @@ class TestExperimentServerIntegration(IBMTestCase):
     def test_retrieve_backends(self):
         """Test retrieving all backends."""
         backends = self.service.backends()
-        self.assertIn(self.backend.name(), [b["name"] for b in backends])
+        self.assertIn(self.backend.name, [b["name"] for b in backends])
 
     def test_create_figure(self):
         """Test creating a figure."""
@@ -1205,6 +1329,30 @@ class TestExperimentServerIntegration(IBMTestCase):
                 expr = self.service.experiment(expr_id)
                 self.assertIn(name, expr.figure_names)
 
+    def test_create_multiple_figures(self):
+        """Test creating multiple figures at once."""
+        file_names = []
+        figure_names = []
+        for name in ["hello world", "another test"]:
+            bytes_data = str.encode(name)
+            words = name.split(" ")
+            file_name = f"{words[0]}_{words[1]}.svg"
+            file_names.append(file_name)
+            figure_names.append(f"{words[0]}.svg")
+            with open(file_names[-1], "wb") as file:
+                file.write(bytes_data)
+            self.assertTrue(
+                os.path.isfile(file_name), f"File {file_name} was not created"
+            )
+            self.addCleanup(os.remove, file_name)
+
+        expr_id = self._create_experiment()
+        figure_list = zip(file_names, figure_names)
+        self.service.create_figures(experiment_id=expr_id, figure_list=figure_list)
+        expr = self.service.experiment(expr_id)
+        for name in figure_names:
+            self.assertIn(name, expr.figure_names)
+
     def test_figure(self):
         """Test getting a figure."""
         hello_bytes = str.encode("hello world")
@@ -1365,14 +1513,13 @@ class TestExperimentServerIntegration(IBMTestCase):
     ) -> str:
         """Create a new experiment."""
         experiment_type = experiment_type or "qiskit_test"
-        backend_name = backend_name or self.backend.name()
+        backend_name = backend_name or self.backend.name
         exp_id = self.service.create_experiment(
             ExperimentData(
                 experiment_type=experiment_type,
                 backend=backend_name,
                 **kwargs,
             ),
-            provider=self.provider,
             json_encoder=json_encoder,
         )
         self.experiments_to_delete.append(exp_id)
@@ -1403,7 +1550,7 @@ class TestExperimentServerIntegration(IBMTestCase):
 
     def _find_backend_device_components(self, min_components):
         """Find a backend with the minimum number of device components."""
-        backend_name = self.backend.name()
+        backend_name = self.backend.name
         device_components = self.device_components
         if len(device_components) < min_components:
             all_components = self.service.device_components()
